/*------------ by Tom Murphy 7  ][  October 2001 ------------*/
/* see multikick.hpp for documentation */

#include "multikick.hpp"

MultiKick::MultiKick (const CRect &size,
                      CControlListener *listener,
                      long tag,
                      int ns,
                      long heightOfOneImage,
                      CBitmap *background,
                      CPoint  &offset,
                      long style,
		      int as) : CControl (size, listener, tag, background),
				numstates(ns),
				offset (offset),
				heightOfOneImage (heightOfOneImage), 
				style(style),
				buttondown(false), 
				obdown(false), 
				actualstate(0), 
				oactualstate(0) {
  if (as == 0) numactualstates = numstates;
  else numactualstates = as;
  setDirty(true); 
}

MultiKick::~MultiKick () {}

/* return the value. Unlike everywhere else, use
   the 'numactualstates' so that we scale to the right
   range. */
float MultiKick::getValue() const
{
  if (numactualstates == 1 || actualstate <= 0)
    return 0.0f;
  if (actualstate >= numactualstates)
    return 1.0f;

  return ((float)actualstate)/((float)(numactualstates-1)) + .0001f;
}

void MultiKick::setValue(float f) {
  actualstate = (int) ((f + .0001f) * (numactualstates-1));
  if (actualstate >= numstates) actualstate = numstates - 1;
}

bool MultiKick::isDirty() {
  return ((actualstate != oactualstate) 
	  || (buttondown != obdown)
	  || CView::isDirty());
}

void MultiKick::setDirty(const bool val) {
  if (val) oactualstate = -1;
  else {
    oactualstate = actualstate;
    obdown = buttondown;
  }
  CView::setDirty(val);
}

void MultiKick::draw (CDrawContext *pContext)
{
  CPoint where (offset.x, offset.y);

  if (style == kKickPairs)
  {
    where.y += heightOfOneImage * ((actualstate<<1) / 2);
    if (buttondown)
      where.x += (size.right - size.left);
  }
  else
  {
    where.y += heightOfOneImage * (actualstate<<1);
    if (buttondown)
      where.y += heightOfOneImage;
  }

    if (getBackground())
    {
        getBackground()->draw(pContext, size, where);
    }

  setDirty (false);
}

//------------------------------------------------------------------------

CMouseEventResult MultiKick::onMouseDown (CPoint& where, const CButtonState& buttons)
{
    int direction;
    
    if (buttons.isLeftButton())
        direction = 1;
    
    if (buttons.isRightButton())
        direction = -1;
    
    int entrystate = actualstate;
    
    beginEdit();
    setState((entrystate + direction + numstates) % numstates);
    setMouseDown(true);
    valueChanged();
    if (isDirty())
        invalid();
    
    endEdit();
    
    return kMouseEventHandled;    
    
}
//------------------------------------------------------------------------

CMouseEventResult MultiKick::onMouseMoved (CPoint& where, const CButtonState& buttons)
{
    return kMouseEventHandled;
}
//------------------------------------------------------------------------

CMouseEventResult MultiKick::onMouseUp (CPoint& where, const CButtonState& buttons)
{
    setMouseDown(false);
    invalid();
    return kMouseEventHandled;
}
//------------------------------------------------------------------------
